@@ -19,7 +19,7 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 # USA
 
-from ..types import override
+from ..overrides import override
 from ..importer import modules
 
 Gdk = modules['Gdk'].introspection_module
